SystemOrganization addCategory: #Gofer!

Object subclass: #Gofer
	instanceVariableNames: 'actions repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Gofer'!

!Gofer methodsFor: 'accessing' stamp: 'lr 7/5/2009 10:13'!
actions
	^ actions! !

!Gofer methodsFor: 'private' stamp: 'lr 7/5/2009 09:56'!
basicCleanup
	MCFileBasedRepository flushAllCaches.
	MCWorkingCopy changed: #allManagers! !

!Gofer methodsFor: 'private' stamp: 'lr 7/5/2009 09:55'!
basicExecute
	| loader |
	loader := MCMultiPackageLoader new.
	actions do: [ :action | action preExecute ].
	actions do: [ :action | action execute: loader ].
	loader load.
	actions do: [ :action | action postExecute ]! !

!Gofer methodsFor: 'actions' stamp: 'lr 7/5/2009 10:40'!
execute
	"Perform the selected load and unload actions."

	actions isEmpty
		ifTrue: [ self error: 'No load/unload actions specified' ].
	self basicExecute; basicCleanup! !

!Gofer methodsFor: 'initialization' stamp: 'lr 7/4/2009 16:59'!
initialize
	actions := OrderedCollection new! !

!Gofer methodsFor: 'actions' stamp: 'lr 7/4/2009 20:31'!
load: aString
	"Install the package named aString in the image."

	actions addLast: (GoferLoad on: self name: aString)! !

!Gofer methodsFor: 'accessing' stamp: 'lr 7/4/2009 18:06'!
repository
	^ repository! !

!Gofer methodsFor: 'options' stamp: 'lr 7/5/2009 10:20'!
repository: aRepository
	"Set the reposiory aRepository to load from during the following load actions."

	MCRepositoryGroup default
		addRepository: aRepository.
	repository := MCRepositoryGroup default repositories
		detect: [ :each | each = aRepository ]
		ifNone: [ nil ]! !

!Gofer methodsFor: 'actions' stamp: 'lr 7/4/2009 20:31'!
unload: aString
	"Remove the package named aString from the image."

	actions addLast: (GoferUnload on: self name: aString)! !

!Gofer methodsFor: 'options' stamp: 'lr 7/5/2009 09:58'!
url: anUrlString
	"Convenience method to set a repository URL using anUrlString."

	self repository: (MCHttpRepository
		location: anUrlString
		user: String new
		password: String new)! !

Object subclass: #GoferAction
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Gofer'!

!GoferAction class methodsFor: 'instance creation' stamp: 'lr 7/4/2009 17:02'!
on: aGopher name: aString
	^ self basicNew initializeOn: aGopher name: aString! !

!GoferAction methodsFor: 'actions' stamp: 'lr 7/4/2009 17:09'!
execute: aLoader
	self subclassResponsibility! !

!GoferAction methodsFor: 'initialization' stamp: 'lr 7/4/2009 17:29'!
initializeOn: aGopher name: aString
	name := aString! !

!GoferAction methodsFor: 'accessing' stamp: 'lr 7/4/2009 18:13'!
name
	^ name! !

!GoferAction methodsFor: 'actions' stamp: 'lr 7/4/2009 17:18'!
postExecute! !

!GoferAction methodsFor: 'actions' stamp: 'lr 7/4/2009 17:29'!
preExecute! !

GoferAction subclass: #GoferLoad
	instanceVariableNames: 'repository version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Gofer'!

!GoferLoad class methodsFor: 'instance creation' stamp: 'lr 7/5/2009 10:14'!
on: aGopher version: aVersion
	^ self new initializeOn: aGopher version: aVersion! !

!GoferLoad methodsFor: 'private' stamp: 'lr 7/5/2009 10:15'!
addDependenciesTo: aGofer
	self version allDependenciesDo: [ :each |
		aGofer actions addLast: (self class
			on: aGofer version: each) ]! !

!GoferLoad methodsFor: 'actions' stamp: 'lr 7/5/2009 10:25'!
execute: aLoader
	aLoader updatePackage: self package withSnapshot: self snapshot! !

!GoferLoad methodsFor: 'private' stamp: 'lr 7/5/2009 10:21'!
findLatestVersionIn: aRepository
	| versions |
	versions := aRepository allVersionNames
		select: [ :each | each beginsWith: self name ].
	versions := versions asSortedCollection: [ :a :b |
		(a copyAfterLast: $.) asNumber <= (b copyAfterLast: $.) asNumber ].
	versions isEmpty
		ifTrue: [ self error: 'No version named ' , self name printString , ' found' ].
	^ aRepository loadVersionFromFileNamed: versions last , '.mcz'! !

!GoferLoad methodsFor: 'initialization' stamp: 'lr 7/5/2009 10:22'!
initializeOn: aGopher name: aString
	super initializeOn: aGopher name: aString.
	self 
		initializeOn: aGopher 
		version: (self findLatestVersionIn: aGopher repository).
	self addDependenciesTo: aGopher! !

!GoferLoad methodsFor: 'initialization' stamp: 'lr 7/5/2009 10:13'!
initializeOn: aGopher version: aVersion
	repository := aGopher repository.
	version := aVersion! !

!GoferLoad methodsFor: 'accessing' stamp: 'lr 7/5/2009 10:25'!
package
	^ self workingCopy package! !

!GoferLoad methodsFor: 'actions' stamp: 'lr 7/4/2009 20:52'!
postExecute
	self workingCopy repositoryGroup 
		addRepository: self repository.
	self workingCopy loaded: self version! !

!GoferLoad methodsFor: 'accessing' stamp: 'lr 7/4/2009 18:32'!
repository
	^ repository! !

!GoferLoad methodsFor: 'accessing' stamp: 'lr 7/5/2009 10:25'!
snapshot
	^ self version snapshot! !

!GoferLoad methodsFor: 'accessing' stamp: 'lr 7/4/2009 18:24'!
version
	^ version! !

!GoferLoad methodsFor: 'accessing' stamp: 'lr 7/4/2009 20:49'!
workingCopy
	^ self version workingCopy! !

GoferAction subclass: #GoferUnload
	instanceVariableNames: 'workingCopy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Gofer'!

!GoferUnload class methodsFor: 'instance creation' stamp: 'lr 7/5/2009 10:40'!
on: aGopher workingCopy: aWorkingCopy
	^ self new initializeOn: aGopher workingCopy: aWorkingCopy! !

!GoferUnload methodsFor: 'private' stamp: 'lr 7/5/2009 10:39'!
addDependenciesTo: aGofer
	| index dependencies |
	index := 1.
	dependencies := OrderedCollection new.
	dependencies addAll: (self workingCopy requiredPackages
		collect: [ :each | each workingCopy ]).
	[ index between: 1 and: dependencies size ] whileTrue: [
		(dependencies at: index) requiredPackages do: [ :each |
			(dependencies includes: each workingCopy)
				ifFalse: [ dependencies addLast: each workingCopy ] ].
		index := index + 1 ].
	dependencies do: [ :each |
		aGofer actions addLast: (self class
			on: aGofer workingCopy: each) ]! !

!GoferUnload methodsFor: 'private' stamp: 'lr 7/4/2009 17:44'!
cleanupCategories
	"Cleanup class categories, leftover from the unloading process."

	self packageInfo systemCategories do: [ :category |
		(SystemOrganization classesInCategory: category) isEmpty
			ifTrue: [ SystemOrganization removeSystemCategory: category ] ]! !

!GoferUnload methodsFor: 'private' stamp: 'lr 7/4/2009 17:44'!
cleanupProtocols
	"Cleanup the method protocols, left over from method extensions."

	self packageInfo foreignClasses do: [ :class |
		(self packageInfo foreignExtensionCategoriesForClass: class) do: [ :category |
			(class organization listAtCategoryNamed: category) isEmpty
				ifTrue: [ class organization removeCategory: category ] ] ]! !

!GoferUnload methodsFor: 'actions' stamp: 'lr 7/4/2009 17:14'!
execute: aLoader
	aLoader unloadPackage: self package! !

!GoferUnload methodsFor: 'initialization' stamp: 'lr 7/5/2009 10:29'!
initializeOn: aGopher name: aString
	super initializeOn: aGopher name: aString.
	self initializeOn: aGopher workingCopy: (MCWorkingCopy registry
		detect: [ :each | each packageName = aString ]
		ifNone: [ self error: 'Working copy ' , aString printString , ' not found' ]).
	self addDependenciesTo: aGopher! !

!GoferUnload methodsFor: 'initialization' stamp: 'lr 7/5/2009 10:29'!
initializeOn: aGopher workingCopy: aWorkingCopy
	workingCopy := aWorkingCopy! !

!GoferUnload methodsFor: 'accessing' stamp: 'lr 7/4/2009 17:36'!
package
	^ self workingCopy package! !

!GoferUnload methodsFor: 'accessing' stamp: 'lr 7/4/2009 17:36'!
packageInfo
	^ self workingCopy packageInfo! !

!GoferUnload methodsFor: 'actions' stamp: 'lr 7/4/2009 18:04'!
postExecute
	self cleanupCategories.
	self cleanupProtocols.
	self unregisterWorkingCopy.
	self unregisterPackageInfo.
	self unregisterRepositories! !

!GoferUnload methodsFor: 'actions' stamp: 'lr 7/4/2009 17:27'!
preExecute
	self packageInfo classes do: [ :class |
		(class selectors includes: #unload)
			ifTrue: [ class unload ] ]! !

!GoferUnload methodsFor: 'private' stamp: 'lr 7/4/2009 17:38'!
unregisterPackageInfo
	"Unregister the package information from the system."

	PackageOrganizer default
		unregisterPackage: self packageInfo! !

!GoferUnload methodsFor: 'private' stamp: 'lr 7/4/2009 19:02'!
unregisterRepositories
	"Remove the repositories if no longer in use by any of the packages."

	self workingCopy repositoryGroup repositories allButFirst do: [ :repository |
		MCWorkingCopy allManagers do: [ :copy |
			(copy repositoryGroup includes: repository)
				ifTrue: [ ^ self ] ].
		MCRepositoryGroup default
			removeRepository: repository ]! !

!GoferUnload methodsFor: 'private' stamp: 'lr 7/4/2009 17:52'!
unregisterWorkingCopy
	"Unregister the working copy."

	MCWorkingCopy registry
		removeKey: self package
		ifAbsent: [ ] ! !

!GoferUnload methodsFor: 'accessing' stamp: 'lr 7/4/2009 17:36'!
workingCopy
	^ workingCopy! !

TestCase subclass: #GoferTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Gofer'!

!GoferTest methodsFor: 'testing' stamp: 'lr 7/4/2009 21:05'!
testLoad
	Gofer new
		url: 'http://source.lukas-renggli.ch/flair';
		load: 'Flair'! !

!GoferTest methodsFor: 'testing' stamp: 'lr 7/4/2009 21:05'!
testUnload
	Gofer new
		unload: 'Flair';
		execute! !
